/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2020-2023, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** JO           Julia Ott                                                     **
** BG           Blandine Guillot                                              **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2020-03-02, JO:   Initial version of revision history              **
** V1.0.1: 2022-01-21, JO:   EP-934: Updated copyright and branding           **
** V1.0.2: 2023-08-21, BG:   EP-1195: Updated compiler version to ARMCC V6    **
**                           Added missing function declarations              **
** V1.0.3: 2023-10-10, BG:   EP-1470: Changed if-conditions into switch       **
*******************************************************************************/

/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** ADC1: Display Poti position with LEDs, running ADC1 in ESM                 **
********************************************************************************
** Read Poti voltage with ADC1-Ch7, measuring Ch7 in ESM.                     **
** The ESM is triggered by Timer13 (CCU6) with a 1ms rate.                    **
** The voltage measured on Ch7 is divided in 7 intervals and displayed        **
** accordingly on the 7 LEDs.                                                 **
** On TLE9855 EvalKit: LED3 to LED9                                           **
** On TLE985x EvalBoard: LED5 to LED11                                        **
*******************************************************************************/         

#include "tle_device.h"

void Poti_Handler(void);

int main(void)
{
  /*****************************************************************************
  ** Initialization of the hardware modules based on the configuration set    **
  ** in Config Wizard                                                         **
  *****************************************************************************/
  TLE_Init();

  ADC1_ESM_En();
  CCU6_StartTmr_T13();

  while (1)
  {
    WDT1_Service();
  }
}

void Poti_Handler(void)
{
#define Interval_mV 625
  uint16 mV;

  bool value_valid;

  value_valid = ADC1_GetChResult_mV(&mV, ADC1_CH7);

  if (value_valid == true)
  {
    switch (mV / Interval_mV)
		{
			case 0:
				PORT_P01_Output_Low_Set();
				PORT_P02_Output_Low_Set();
				PORT_P14_Output_Low_Set();
				PORT_P03_Output_Low_Set();
				PORT_P04_Output_Low_Set();
				PORT_P05_Output_Low_Set();
				PORT_P12_Output_Low_Set();
				break;
			case 1:
				PORT_P01_Output_High_Set();
				PORT_P02_Output_Low_Set();
				PORT_P14_Output_Low_Set();
				PORT_P03_Output_Low_Set();
				PORT_P04_Output_Low_Set();
				PORT_P05_Output_Low_Set();
				PORT_P12_Output_Low_Set();
				break;
			case 2:
				PORT_P01_Output_High_Set();
				PORT_P02_Output_High_Set();
				PORT_P14_Output_Low_Set();
				PORT_P03_Output_Low_Set();
				PORT_P04_Output_Low_Set();
				PORT_P05_Output_Low_Set();
				PORT_P12_Output_Low_Set();
				break;
			case 3:
				PORT_P01_Output_High_Set();
				PORT_P02_Output_High_Set();
				PORT_P14_Output_High_Set();
				PORT_P03_Output_Low_Set();
				PORT_P04_Output_Low_Set();
				PORT_P05_Output_Low_Set();
				PORT_P12_Output_Low_Set();
				break;
			case 4:
				PORT_P01_Output_High_Set();
				PORT_P02_Output_High_Set();
				PORT_P14_Output_High_Set();
				PORT_P03_Output_High_Set();
				PORT_P04_Output_Low_Set();
				PORT_P05_Output_Low_Set();
				PORT_P12_Output_Low_Set();
				break;
			case 5:
				PORT_P01_Output_High_Set();
				PORT_P02_Output_High_Set();
				PORT_P14_Output_High_Set();
				PORT_P03_Output_High_Set();
				PORT_P04_Output_High_Set();
				PORT_P05_Output_Low_Set();
				PORT_P12_Output_Low_Set();
				break;
			case 6:
				PORT_P01_Output_High_Set();
				PORT_P02_Output_High_Set();
				PORT_P14_Output_High_Set();
				PORT_P03_Output_High_Set();
				PORT_P04_Output_High_Set();
				PORT_P05_Output_High_Set();
				PORT_P12_Output_Low_Set();
				break;
			default:
				PORT_P01_Output_High_Set();
				PORT_P02_Output_High_Set();
				PORT_P14_Output_High_Set();
				PORT_P03_Output_High_Set();
				PORT_P04_Output_High_Set();
				PORT_P05_Output_High_Set();
				PORT_P12_Output_High_Set();
				break;
    }
  }
}
