/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2020-2023, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** JO           Julia Ott                                                     **
** BG           Blandine Guillot                                              **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2020-03-02, JO:   Initial version of revision history              **
** V1.0.1: 2022-01-21, JO:   EP-934: Updated copyright and branding           **
** V1.0.2: 2023-08-21, BG:   EP-1195: Updated compiler version to ARMCC V6    **
**                           Added missing function declarations              **
*******************************************************************************/

/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** ADC2: VS over- and undervoltage detection                                  **
********************************************************************************
** Enable VS over- and undervoltage interrupt.                                **
** LED_P0.2 will be on in case of VS undervoltage.                            **
** LED_P0.1 will be on in case of VS overvoltage.                             **
*******************************************************************************/

#include "tle_device.h"

void VS_Over_Voltage_Diag(void);
void VS_Under_Voltage_Diag(void);
void Diagnostic_Status_Clear(void);

int main(void)
{
  /*****************************************************************************
  ** Initialization of the hardware modules based on the configuration set    **
  ** in Config Wizard                                                         **
  *****************************************************************************/
  TLE_Init();

  while (1)
  {
    WDT1_Service();
    Diagnostic_Status_Clear();
  }
}

/* Callback function for ADC2 VS Upper Threshold Interrupt */
void VS_Over_Voltage_Diag(void)
{
  PORT_P01_Output_High_Set();
}

/* Callback function for ADC2 VS Lower Threshold Interrupt */
void VS_Under_Voltage_Diag(void)
{
  PORT_P02_Output_High_Set();
}

void Diagnostic_Status_Clear(void)
{
  /* Check for VS overvoltage status */
  if (SCUPM->SYS_SUPPLY_IRQ_STS.bit.VS_OV_STS == 1u)
  {
    /* Clear VS overvoltage status */
    SCUPM->SYS_SUPPLY_IRQ_CLR.bit.VS_OV_SC = 1u;

    /* Status set again if overvoltage is still present */
    /* Status remains clear if overvoltage is gone */
    if (SCUPM->SYS_SUPPLY_IRQ_STS.bit.VS_OV_STS == 0u)
    {
      /* If status stays clear, no more overvoltage condition and LED can be cleared */
      PORT_P01_Output_Low_Set();
    }
  }
  
  /* Check for VS undervoltage status*/
  if (SCUPM->SYS_SUPPLY_IRQ_STS.bit.VS_UV_STS == 1u)
  {
    /* Clear VS undervoltage status */
    SCUPM->SYS_SUPPLY_IRQ_CLR.bit.VS_UV_SC = 1u;

    /* Status set again if undervoltage is still present */
    /* Status remains cleared if undervoltage is gone */
    if (SCUPM->SYS_SUPPLY_IRQ_STS.bit.VS_UV_STS == 0u)
    {
      /* If status stays clear, no more undervoltage condition and LED can be cleared */
      PORT_P02_Output_Low_Set();
    }
  }
}
