/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2020-2023, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** JO           Julia Ott                                                     **
** BG           Blandine Guillot                                              **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2020-03-02, JO:   Initial version of revision history              **
** V1.0.1: 2022-01-21, JO:   EP-934: Updated copyright and branding           **
** V1.0.2: 2023-08-21, BG:   EP-1195: Updated compiler version to ARMCC V6    **
**                           Added missing function declarations              **
**                           Updated type conversion                          **
*******************************************************************************/

/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** Running lights                                                             **
********************************************************************************
** LEDs are turned on then off, one after another, continuously.              **
*******************************************************************************/

#include "tle_device.h"

int PWM (int state, int position);
void PWM1_LEDsOn(void);
void PWM2_LEDsOn(void);
void PWM3_LEDsOn(void);
void PWM_PM_AllOff(void);

#define LED3 0x01
#define LED4 0x02
#define LED5 0x14
#define LED6 0x03
#define LED7 0x04
#define LED8 0x05
#define LED9 0x12

/* Number of LEDs */
#define N_LED 7u

static int i = 0; /* step duration = (PWMPeriod * i) */
static int x = 1; /* sign for PWM calc. matrix state - x * position */
static int LED[N_LED] = {LED3, LED4, LED5, LED6, LED7, LED8, LED9};
static int state = 0;

int main(void)
{
  /*****************************************************************************
  ** Initialization of the hardware modules based on the configuration set    **
  ** in Config Wizard                                                         **
  *****************************************************************************/
  TLE_Init();

  /* Start Timer12 */
  CCU6_StartTmr_T12();

  for (;;)
  {
    (void)WDT1_Service();

  }
}


int PWM (int sts, int position)
{
  int pwm = 0;
  
  if (((sts - x * position) == -1) || ((sts - x * position) == 13))
  {
    pwm = 1;
  }
  
  if (((sts - x * position) == 0) || ((sts - x * position) == 14))
  {
    pwm = 2;
  }
  
  if (((sts - x * position) == 1) || ((sts - x * position) == 15))
  {
    pwm = 3;
  }
  
  return pwm;
}


void PWM1_LEDsOn(void)
{  
  int y = 0;

  for (y = 0; y < (int)N_LED; y++)
  {
    if (PWM(state, y) == 1)
    {
      PORT_ChangePin ((uint32)LED[y], PORT_ACTION_SET);
    }
  }
}
  
  
void PWM2_LEDsOn(void)
{  
  int y = 0;
  
  for (y = 0; y < (int)N_LED; y++)
  {
    if (PWM(state, y) == 2)
    {
      PORT_ChangePin ((uint32)LED[y], PORT_ACTION_SET);
    }
  }
}
  
  
void PWM3_LEDsOn(void)
{  
  int y = 0;
  
  for (y = 0; y < (int)N_LED; y++)
  {
    if (PWM (state, y) == 3)
    {
      PORT_ChangePin ((uint32)LED[y], PORT_ACTION_SET);
    }
  }
}


void PWM_PM_AllOff(void)
{ 
  PORT->P0_DATA.reg = 0;
  PORT->P1_DATA.reg = 0;
  
  i++;
  
  if (i == 60)
  {  
    state++;
    i = 0;
  }
  
  if (state == 7)
  {
    x = -1;
  }
  
  if (state == 14)
  {
    x = 1;
    state = 0;
  }
}
