/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2017, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** DM           Daniel Mysliwitz                                              **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2014-05-20, DM:   Initial version                                  **
*******************************************************************************/

#include "linmain.h"
#include <timer2x.h>

#if defined(__ARMCC_VERSION) && (__ARMCC_VERSION >= 6000000)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wmissing-variable-declarations"
#endif
volatile unsigned char timer_250us, timer_1ms;
#if defined(__ARMCC_VERSION) && (__ARMCC_VERSION >= 6000000)
#pragma clang diagnostic pop
#endif


void LIN_LLD_Init(void)
{
	TIMER21_Start();
	
	/* LIN driver init */
	l_sys_init();
}

void LIN_LLD_Service(void)
{
	if(timer_250us)
	{
		timer_250us = 0;
		
		/* Cyclic communication task has to be called every byte */
		l_cyclic_com_task();
	}
	
	/* 1ms cyclic task */
	if(timer_1ms >= 4)
	{
		timer_1ms = 0;
		l_sch_tick();
		
		/* Cyclic task of driver (timeout and diagnostic control) */
		ld_task();
	}
}


l_irqmask l_sys_irq_disable (void)
{
   return 0u;
}


#if defined(__ARMCC_VERSION) && (__ARMCC_VERSION >= 6000000)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
#endif
void l_sys_irq_restore (l_irqmask mask)
{
   return;
}
#if defined(__ARMCC_VERSION) && (__ARMCC_VERSION >= 6000000)
#pragma clang diagnostic pop
#endif


/* BEGIN: Only used for slaves */
void LIN_Enable_Transceiver(void)
{
}

void LIN_Disable_Transceiver(void) 
{
}
/* END: Only used for slaves */


/* Callback is called every 250us */
void LIN_Task_250us(void)
{
	timer_250us++;
	timer_1ms++;
}


void LIN_COM_Handler(void)
{
#ifdef AUTOBAUD_ENABLED
   if (TIMER2->T2CON.bit.EXF2 == 1u)
   {
      l_ifc_t2();
      TIMER2->T2ICLR.bit.EXF2CLR = 1u;
   }
#endif /* end #ifdef AUTOBAUD_ENABLED */
   if ((UART1->SCON.reg & 0x01u) == 1u)
   {
      l_ifc_rx();
      UART1->SCONCLR.bit.RICLR = 1u;
   }
}
