/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2020-2023, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** JO           Julia Ott                                                     **
** BG           Blandine Guillot                                              **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2020-03-02, JO:   Initial version of revision history              **
** V1.0.1: 2022-01-21, JO:   EP-934: Updated copyright and branding           **
** V1.0.2: 2023-08-21, BG:   EP-1195: Updated compiler version to ARMCC V6    **
**                           Added missing function declarations              **
*******************************************************************************/

/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** PMU: VDDEXT enable and diagnostic function                                 **
********************************************************************************
** VDDEXT enabled (5V at VDDEXT pin)                                          **
** LED_P1_2 will be set upon undervoltage at VDDEXT.                          **
** LED_P1_4 will be set upon Over Temperature at VDDEXT.                      **
** Pressing MON1 button clears LEDs.                                          **
*******************************************************************************/

#include "tle_device.h"

void VDDEXT_Under_Volt_Diag_Callback(void);
void VDDEXT_Over_Temp_Diag_Callback(void);

static uint8 bVDDEXT_Fail;

int main(void)
{
  /*****************************************************************************
  ** Initialization of the hardware modules based on the configuration set    **
  ** in Config Wizard                                                         **
  *****************************************************************************/
  TLE_Init();

  /* Initialize VDDEXT fail indication */
  bVDDEXT_Fail = 0;

  for (;;)
  {
    (void)WDT1_Service();

    /* Check if VDDEXT has failed */
    if (bVDDEXT_Fail == 1u)
    {
      /* If MON1 button is pushed, clear VDDEXT error */
      if(MON1_Sts() == false)
      {
        /* Reset error LEDs */
        PORT_P12_Output_Low_Set();
        PORT_P14_Output_Low_Set();
        /* Clear VDDEXT error flags */
        PMU_VDDEXT_Error_Clear(PMU_VDDEXT_OT_SC);
        /* Reset VDDEXT fail indication variable */
        bVDDEXT_Fail = 0u;
      }
    }
  }
}

/* Callback function on undervoltage diagnosis */
void VDDEXT_Under_Volt_Diag_Callback(void)
{
  /* Apply a delay of approx. 50us to give error flags in VDDEXT_CTRL time to show up */
  Delay_us(50);
  /* Set error LED_P1_2 */
  PORT_P12_Output_High_Set();
  /* Set VDDEXT fail indication variable */
  bVDDEXT_Fail = 1u;
}

/* Callback function on over-temperature diagnosis */
void VDDEXT_Over_Temp_Diag_Callback(void)
{
  /* Apply a delay of approx. 50us to give error flags in VDDEXT_CTRL time to show up */
  Delay_us(50);
  /* Set error LED_P1_4 */
  PORT_P14_Output_High_Set();
  /* Set VDDEXT fail indication variable */
  bVDDEXT_Fail = 1u;
}
