/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2020-2023, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** JO           Julia Ott                                                     **
** BG           Blandine Guillot                                              **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2020-03-02, JO:   Initial version of revision history              **
** V1.0.1: 2022-01-21, JO:   EP-934: Updated copyright and branding           **
** V1.0.2: 2023-08-21, BG:   EP-1195: Updated compiler version to ARMCC V6    **
**                           Added missing function declarations              **
*******************************************************************************/

/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** SSC1/SSC2: Send data from SSC1 to SSC2                                     **
********************************************************************************
** Both SSC operate at 1Mbaud. SSC1 sends one byte at P0.3 (CLK), P0.4 (MTSR).**
** SSC2 receives one byte at P1.0 (CLK), P1.1 (MTSR). Connect clock signals   **
** and data signal from SSC1 and SSC2 together.                               **
** P1.2 is set on receive.                                                    **
** (P1.2 is LED9 on TLE9855 EvalKit/LED5 in TLE985x EvalBoard)                **
*******************************************************************************/

#include "tle_device.h"

void Data_Received(void);

int main(void)
{
  /*****************************************************************************
  ** Initialization of the hardware modules based on the configuration set    **
  ** in Config Wizard                                                         **
  *****************************************************************************/
  TLE_Init();

  SSC1_SendWord((uint16) 0xAAAA);

  for (;;)
  {
    (void)WDT1_Service();
  }
}

void Data_Received(void)
{
  uint16 data = SSC2_ReadWord();
  if (data == (uint16) 0x00AA)
  {
    PORT_P12_Output_High_Set();
  }
}
