;**********************************************************************************************************************
;
; @file     startup_tle985x.s
; @brief    CMSIS Cortex-M0 Core Device Startup File for
;           the Infineon TLE985x Device Series
; @version  V1.1.0
; @date     19. Jan 2022
; 
; Copyright (c) Infineon Technologies AG
; All rights reserved.
;
; Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
; following conditions are met:
;
;   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
;   disclaimer.
;
;   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
;   following disclaimer in the documentation and/or other materials provided with the distribution.
;
;   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
;   products derived from this software without specific prior written permission.
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
; INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
; DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
; SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
; SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
; WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
; USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;
;*********************************************************************************************************************/

;*******************************************************************************
;                              Author(s) Identity                             **
;*******************************************************************************
;  Initials     Name                                                          **
;  ---------------------------------------------------------------------------**
;  BG           Blandine Guillot                                              **
;******************************************************************************/

;*******************************************************************************
;                           Revision Control History                          **
;*******************************************************************************
;  V1.1.0: 2022-01-19, BG:   Initial version                                  **
;******************************************************************************/

        MODULE  ?vector_table

        AAPCS INTERWORK, VFP_COMPATIBLE, RWPI_COMPATIBLE
        PRESERVE8


        ;; Forward declaration of sections.
        SECTION CSTACK:DATA:NOROOT(3)

        SECTION .intvec:CODE:NOROOT(2)

        EXTERN  __iar_program_start
        EXTERN  SystemInit
        PUBLIC  __vector_table

        DATA

; The vector table is not needed until after copy initialization is done
__iar_init$$done:                         

__vector_table
        DCD     sfe(CSTACK)
        DCD     Reset_Handler               ; Reset Handler
        DCD     NMI_Handler                 ; NMI Handler
        DCD     HardFault_Handler           ; Hard Fault Handler
        DCD     MemManage_Handler           ; MPU Fault Handler
        DCD     BusFault_Handler            ; Bus Fault Handler
        DCD     UsageFault_Handler          ; Usage Fault Handler
        DCD     0                           ; Reserved
        DCD     0                           ; Reserved
        DCD     0                           ; Reserved
        DCD     0                           ; Reserved
        DCD     SVC_Handler                 ; SVCall Handler
        DCD     DebugMon_Handler            ; Debug Monitor Handler
        DCD     0                           ; Reserved
        DCD     PendSV_Handler              ; PendSV Handler
        DCD     SysTick_Handler             ; SysTick Handler
                
; External Interrupts
        DCD     GPT1_IRQHandler             ; 0:  GPTimer Interrupt 1
        DCD     GPT2_IRQHandler             ; 1:  GPTimer Interrupt 2
        DCD     ADC2_IRQHandler             ; 2:  MU interrupt / ADC2, VBG interrupt
        DCD     ADC1_IRQHandler             ; 3:  ADC 10 Bit interrupt
        DCD     CCU6SR0_IRQHandler          ; 4:  CCU6 node 0 interrupt
        DCD     CCU6SR1_IRQHandler          ; 5:  CCU6 node 1 interrupt
        DCD     CCU6SR2_IRQHandler          ; 6:  CCU6 node 2 interrupt
        DCD     CCU6SR3_IRQHandler          ; 7:  CCU6 node 3 interrupt
        DCD     SSC1_IRQHandler             ; 8:  SSC1 interrupt (receive, transmit, error)
        DCD     SSC2_IRQHandler             ; 9:  SSC2 interrupt (receive, transmit, error)
        DCD     UART1_IRQHandler            ; 10: UART1 (ASC-LIN) interrupt (receive, transmit), T2, linsync1, LIN
        DCD     UART2_IRQHandler            ; 11: UART2 interrupt (receive, transmit), T21, External interrupt (EINT2)
        DCD     EXINT0_IRQHandler           ; 12: External interrupt (EINT0), wakeup
        DCD     EXINT1_IRQHandler           ; 13: External interrupt (EINT1)
        DCD     WAKEUP_IRQHandler           ; 14: Wakeup interrupt (generated by a wakeup event)
        DCD     DIV_IRQHandler              ; 15: Hardware divider interrupt
        DCD     0                           ; 16: reserved for future use
        DCD     CP_IRQHandler               ; 17: Charge Pump interrupt
        DCD     BDRV_IRQHandler             ; 18: Bridge Driver interrupt
        DCD     HS_IRQHandler               ; 19: High Side Interrupt
        DCD     CSA_IRQHandler              ; 20: OpAmp Interrupt
        DCD     DU1_IRQHandler              ; 21: Differential Unit - DPP1
        DCD     MONx_IRQHandler             ; 22: MONx Interrupt, wakeup
        DCD     PORT2_IRQHandler            ; 23: Port 2.x - DPP1

Default_Handler
        NOCALL Default_Handler
        B Default_Handler


        SECTION .text:CODE:REORDER:NOROOT(2)
        THUMB
Reset_Handler:
        LDR     R0, =SystemInit           ; Call the device specific initialization (core clock, PLL etc.)
        BLX     R0

        LDR     R0, =__iar_program_start
        BX      R0


        PUBWEAK     NMI_Handler           ; NMI Handler
        SECTION .text:CODE:REORDER:NOROOT(1)
NMI_Handler
        B NMI_Handler


        PUBWEAK     HardFault_Handler     ; Hard Fault Handler
        SECTION .text:CODE:REORDER:NOROOT(1)
HardFault_Handler
        B HardFault_Handler 


        PUBWEAK     MemManage_Handler     ; MPU Fault Handler
        SECTION .text:CODE:REORDER:NOROOT(1)
MemManage_Handler
        B MemManage_Handler 


        PUBWEAK     BusFault_Handler      ; Bus Fault Handler
        SECTION .text:CODE:REORDER:NOROOT(1)
BusFault_Handler
        B BusFault_Handler  


        PUBWEAK     UsageFault_Handler      ; Usage Fault Handler
        SECTION .text:CODE:REORDER:NOROOT(1)
UsageFault_Handler
        B UsageFault_Handler  


        PUBWEAK     SVC_Handler           ; SVCall Handler
        SECTION .text:CODE:REORDER:NOROOT(1)
SVC_Handler
        B SVC_Handler 


        PUBWEAK     DebugMon_Handler      ; Debug Monitor Handler
        SECTION .text:CODE:REORDER:NOROOT(1)
DebugMon_Handler
        B DebugMon_Handler  


        PUBWEAK     PendSV_Handler        ; PendSV Handler
        SECTION .text:CODE:REORDER:NOROOT(1)
PendSV_Handler
        B PendSV_Handler  


        PUBWEAK     SysTick_Handler       ; SysTick Handler
        SECTION .text:CODE:REORDER:NOROOT(1)
SysTick_Handler
        B SysTick_Handler       


        PUBWEAK     GPT1_IRQHandler       ; 0:  GPTimer Interrupt 1
        SECTION .text:CODE:REORDER:NOROOT(1)
GPT1_IRQHandler
        B GPT1_IRQHandler 


        PUBWEAK     GPT2_IRQHandler       ; 1:  GPTimer Interrupt 2
        SECTION .text:CODE:REORDER:NOROOT(1)
GPT2_IRQHandler
        B GPT2_IRQHandler 


        PUBWEAK     ADC2_IRQHandler       ; 2:  MU interrupt / ADC2, VBG interrupt
        SECTION .text:CODE:REORDER:NOROOT(1)
ADC2_IRQHandler
        B ADC2_IRQHandler     


        PUBWEAK     ADC1_IRQHandler       ; 3:  ADC10 Bit interrupt
        SECTION .text:CODE:REORDER:NOROOT(1)
ADC1_IRQHandler
        B ADC1_IRQHandler 


        PUBWEAK     CCU6SR0_IRQHandler    ; 4:  CCU6 node 0 Interrupt
        SECTION .text:CODE:REORDER:NOROOT(1)
CCU6SR0_IRQHandler
        B CCU6SR0_IRQHandler    


        PUBWEAK     CCU6SR1_IRQHandler    ; 5:  CCU6 node 1 Interrupt
        SECTION .text:CODE:REORDER:NOROOT(1)
CCU6SR1_IRQHandler
        B CCU6SR1_IRQHandler


        PUBWEAK     CCU6SR2_IRQHandler    ; 6:  CCU6 node 2 Interrupt
        SECTION .text:CODE:REORDER:NOROOT(1)
CCU6SR2_IRQHandler
        B CCU6SR2_IRQHandler


        PUBWEAK     CCU6SR3_IRQHandler    ; 7:  CCU6 node 3 Interrupt
        SECTION .text:CODE:REORDER:NOROOT(1)
CCU6SR3_IRQHandler
        B CCU6SR3_IRQHandler


        PUBWEAK     SSC1_IRQHandler       ; 8:  SSC1 Interrupt (receive, transmit, error)
        SECTION .text:CODE:REORDER:NOROOT(1)
SSC1_IRQHandler
        B SSC1_IRQHandler


        PUBWEAK     SSC2_IRQHandler       ; 9:  SSC2 Interrupt (receive, transmit, error)
        SECTION .text:CODE:REORDER:NOROOT(1)
SSC2_IRQHandler
        B SSC2_IRQHandler


        PUBWEAK     UART1_IRQHandler      ; 10: UART1 (ASC-LIN) Interrupt (receive, transmit), t2, linsync1, LIN
        SECTION .text:CODE:REORDER:NOROOT(1)
UART1_IRQHandler
        B UART1_IRQHandler


        PUBWEAK     UART2_IRQHandler      ; 11: UART2 Interrupt (receive, transmit), t21, External interrupt (EINT2)
        SECTION .text:CODE:REORDER:NOROOT(1)
UART2_IRQHandler
        B UART2_IRQHandler


        PUBWEAK     EXINT0_IRQHandler     ; 12: External Interrupt (EINT0), wakeup
        SECTION .text:CODE:REORDER:NOROOT(1)
EXINT0_IRQHandler
        B EXINT0_IRQHandler


        PUBWEAK     EXINT1_IRQHandler     ; 13: External Interrupt (EINT1)
        SECTION .text:CODE:REORDER:NOROOT(1)
EXINT1_IRQHandler
        B EXINT1_IRQHandler


        PUBWEAK     WAKEUP_IRQHandler     ; 14: Wakeup Interupt (generated by a wakeup event)
        SECTION .text:CODE:REORDER:NOROOT(1)
WAKEUP_IRQHandler
        B WAKEUP_IRQHandler


        PUBWEAK     DIV_IRQHandler        ; 15: Hardware divider interrupt
        SECTION .text:CODE:REORDER:NOROOT(1)
DIV_IRQHandler
        B DIV_IRQHandler


        PUBWEAK     CP_IRQHandler         ; 17: Charge Pump interrupt
        SECTION .text:CODE:REORDER:NOROOT(1)
CP_IRQHandler
        B CP_IRQHandler


        PUBWEAK     BDRV_IRQHandler       ; 18: Bridge Driver interrupt
        SECTION .text:CODE:REORDER:NOROOT(1)
BDRV_IRQHandler
        B BDRV_IRQHandler


        PUBWEAK     HS_IRQHandler         ; 19: High Side Interrupt
        SECTION .text:CODE:REORDER:NOROOT(1)
HS_IRQHandler
        B HS_IRQHandler


        PUBWEAK     CSA_IRQHandler        ; 20: OpAmp Interrupt
        SECTION .text:CODE:REORDER:NOROOT(1)
CSA_IRQHandler
        B CSA_IRQHandler


        PUBWEAK     DU1_IRQHandler        ; 21: Differential Unit - DPP1
        SECTION .text:CODE:REORDER:NOROOT(1)
DU1_IRQHandler
        B DU1_IRQHandler


        PUBWEAK     MONx_IRQHandler       ; 22: MONx Interrupt, wakeup
        SECTION .text:CODE:REORDER:NOROOT(1)
MONx_IRQHandler
        B MONx_IRQHandler


        PUBWEAK     PORT2_IRQHandler      ; 23: Port 2.x - DPP1
        SECTION .text:CODE:REORDER:NOROOT(1)
PORT2_IRQHandler
        B PORT2_IRQHandler


        END